import LibHanger.Library.uwLogger as Logger
from LibHanger.Library.DataAccess.uwPostgres import uwPostgreSQL
from LibHanger.Models.recset import recset
from LibHanger.Models.saWhere import saWhere
from netkeiber.Getter.prm_traning_ass import getter_prm_traning_ass
from netkeiber.Getter.trn_race_result import getter_trn_race_result
from netkeiber.Getter.trn_horse_result import getter_trn_horse_result
from netkeiber.Getter.trn_open_info import getter_trn_open_info
from netkeiber.Getter.trn_refund_info import getter_trn_refund_info
from netkeiber.Getter.log_race_id import getter_log_race_id
from netkeiber.Models.trn_horse_result import trn_horse_result
from netkeiber.Models.trn_race_info import trn_race_info
from netkeiber.Models.trn_open_info import trn_open_info
from netkeiber.Models.trn_refund_info import trn_refund_info
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Library.netkeiberException import getterError


class getter_RaceData:
    """
    レースデータ取得クラス
    """

    def __init__(self, __psgr: uwPostgreSQL):
        """
        コンストラクタ

        Parameters
        ----------
        __psgr : uwPostgreSQL
            uwPostgreSQLクラスインスタンス
        """

        # uwPostgreSQL
        self.__psgr = __psgr

        # Getterインスタンス生成
        self.createGetterInstance()

        # レコードセット初期化
        self.init_recset()

        # 取得フラグ
        self.raceResultRead = True
        self.traningAssRead = True
        self.horseResultRead = True
        self.openInfoRead = True
        self.refundInfoRead = True

        # エラー有無
        self.__hasError = False

        # スクレイピング回数
        self.__scrapingCount = 0

    def __exit__(self, ex_type, ex_value, trace):
        """
        ContextManger - exit

        Parameters
        ----------
            ex_type : Any
                ex_type
            ex_value : Any
                ex_value
            trace : Any
                trace
        """

        # with内で例外発生時
        if ex_type != None:
            # ログ出力
            Logger.logging.error(f"Exception Type={ex_type}")
            Logger.logging.error(f"Exception Value={ex_value}")
            Logger.logging.error(f"Stack trace={trace}")

        # WebDriver - Quit
        self.raceResult.quitWebDriver()
        self.horseResult.quitWebDriver()
        self.openInfo.quitWebDriver()
        self.refundInfo.quitWebDriver()
        self.traningAssInfo.quitWebDriver()
        self.raceIdLog.quitWebDriver()
        
        # Trueを返して例外を握りつぶす
        return True
    
    def createGetterInstance(self):
        """
        Getterインスタンス生成
        """

        # レース結果取得クラスインスタンス
        self.raceResult = getter_trn_race_result()

        # 競走馬成績取得クラスインスタンス
        self.horseResult = getter_trn_horse_result()

        # 開催情報取得クラスインスタンス
        self.openInfo = getter_trn_open_info()

        # 払戻情報取得クラスインスタンス
        self.refundInfo = getter_trn_refund_info()

        # 調教情報取得クラスインスタンス
        self.traningAssInfo = getter_prm_traning_ass()

        # レースIDログ
        self.raceIdLog = getter_log_race_id()

    def init_recset(self):
        """
        レコードセット初期化
        """

        self.raceResult.init_recset()
        self.horseResult.init_recset()
        self.openInfo.init_recset()
        self.refundInfo.init_recset()
        self.traningAssInfo.init_recset()
        if self.raceResult.wdc.browserCtl != None:
            self.raceResult.wdc.browserCtl.resetLoadPageCount()
        if self.horseResult.wdc.browserCtl != None:
            self.horseResult.wdc.browserCtl.resetLoadPageCount()
        if self.openInfo.wdc.browserCtl != None:
            self.openInfo.wdc.browserCtl.resetLoadPageCount()
        if self.refundInfo.wdc.browserCtl != None:
            self.refundInfo.wdc.browserCtl.resetLoadPageCount()
        if self.traningAssInfo.wdc.browserCtl != None:
            self.traningAssInfo.wdc.browserCtl.resetLoadPageCount()
        self.raceResult.rsRaceResult.setDA(self.__psgr)
        self.horseResult.rsHorseResult.setDA(self.__psgr)
        self.openInfo.rsOpenInfo.setDA(self.__psgr)
        self.refundInfo.rsRefundInfo.setDA(self.__psgr)
        self.traningAssInfo.rsTraningAss.setDA(self.__psgr)

    @property
    def scrapingCount(self):
        """
        スクレイピング回数
        """

        return self.__scrapingCount

    @property
    def hasError(self):
        """
        エラー有無
        """

        return self.__hasError

    def getData(self, race_id, racecourse_id):
        """
        データ取得

        Parameters
        ----------
        race_id : str
            レースID
        racecourse_id : str
            競馬場ID
        """

        try:

            # エラーフラグ初期化
            self.__hasError = False

            # session - open
            self.__psgr.openSession()

            # レース結果取得判定
            if self.raceResultRead:

                # レース結果取得
                self.raceResult.getData(race_id=race_id)

                # レース結果の取得件数が0件だった場合は処理を抜ける
                if self.raceResult.rsRaceResult.recordCount == 0:
                    return

            # 調教情報取得判定
            if self.traningAssRead:

                # 調教情報取得
                self.traningAssInfo.getData(race_id=race_id)

            # saWhereインスタンス
            saw = saWhere()

            # レース結果ループ開始
            rsRaceResult = self.raceResult.rsRaceResult
            rsRaceInfo = self.raceResult.rsRaceInfo
            rsRaceInfo.first()
            while rsRaceResult.eof() == False:

                # 競走馬成績取得
                if self.horseResultRead:
                    
                    # 取得済データチェック
                    saw.clear()
                    saw.and_(trn_horse_result.horse_id == rsRaceResult.fields(trn_horse_result.horse_id).value
                    ).and_(trn_horse_result.race_id == rsRaceResult.fields(trn_horse_result.race_id).value
                    )
                    if self.horseResult.rsHorseResult.find(saw, recset.findOption.dataBase) == False:

                        # 競走馬成績取得
                        self.horseResult.getData(horse_id=rsRaceResult.fields(trn_horse_result.horse_id).value)

                # 払戻情報取得
                if self.refundInfoRead:

                    # 払戻ID取得
                    refund_id = rsRaceInfo.fields(trn_race_info.refund_id).value

                    # 取得済データチェック
                    rsRefundInfo = self.refundInfo.rsRefundInfo
                    saw.clear()
                    saw.and_(trn_refund_info.refund_id == refund_id
                    ).and_(trn_refund_info.racecourse_id == racecourse_id
                    ).and_(trn_refund_info.race_no == rsRaceInfo.fields(trn_race_info.race_no).value
                    )
                    if rsRefundInfo.find(saw, recset.findOption.dataBase) == False:

                        # 払戻情報取得
                        self.refundInfo.getData(refund_id=refund_id, racecourse_id=racecourse_id)

            # 開催情報取得
            if self.openInfoRead:
                dfOpenIdGroup = self.horseResult.rsHorseResult.getDataFrame().groupby(trn_open_info.open_id.key)
                for open_id, _ in dfOpenIdGroup:

                    # 取得済データチェック
                    rsOpenInfo = self.openInfo.rsOpenInfo
                    saw.clear()
                    saw.and_(trn_open_info.open_id == open_id
                    ).and_(trn_open_info.racecourse_id == racecourse_id
                    )
                    if rsOpenInfo.find(saw, recset.findOption.dataBase) == False:

                        # 開催情報取得
                        self.openInfo.getData(open_id=open_id, racecourse_id=racecourse_id)

            # スクレイピング回数取得
            rsList = []
            rsList.append(self.raceResult.scrapingCount)
            rsList.append(self.horseResult.scrapingCount)
            rsList.append(self.openInfo.scrapingCount)
            rsList.append(self.refundInfo.scrapingCount)
            rsList.append(self.traningAssInfo.scrapingCount)
            self.__scrapingCount = sum(rsList)

        except getterError as geException:
            self.__hasError = True
            self.raceIdLog.setLogRaceId(race_id, nd.getterResult.error, geException)
        except Exception as e:
            self.__hasError = True
            self.raceIdLog.setLogRaceId(race_id, nd.getterResult.error, e)
        finally:
            # session - close
            self.__psgr.closeSession()



import urllib.parse as urlParse
import LibHanger.Library.uwLogger as Logger
from enum import Enum
from bs4 import Tag
from pandas.core.frame import DataFrame
from selenium.webdriver.remote.webelement import WebElement
from LibHanger.Models.recset import recset
from Scrapinger.Library.browserContainer import browserContainer
from netkeiber.Library.netkeiberConfig import netkeiberConfig
from netkeiber.Library.netkeiberException import racdIdCheckError
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Models.trn_race_id import trn_race_id
from netkeiber.Getter.Base.baseGetter import baseGetter


class getter_trn_race_id(baseGetter):
    """
    レースID情報取得クラス
    (trn_race_id)
    """

    class methodType(Enum):
        """
        データ取得メソッド
        """

        getOpenCal = 0
        """ 開催カレンダー取得メソッド """

        getRaceId = 1
        """ レースID情報取得メソッド(蓄積系) """

        getPrmRaceId = 2
        """ レースID情報取得メソッド(速報系) """

    def __init__(self) -> None:
        """
        コンストラクタ
        """

        super().__init__()

        # レコードセット初期化
        self.init_recset()

    def init_recset(self):
        """
        レコードセット初期化
        """

        # レコードセット初期化
        self.rsRaceId = recset[trn_race_id](trn_race_id)

    @property
    def scrapingType(self):
        """
        ScrapingType
        スクレイピング方法を指定する(unknownの場合はconfig側で指定した設定を優先する)
        """

        return netkeiberConfig.settingValueStruct.ScrapingType.selenium

    @Logger.loggerDecorator("getData", ["year"])
    def getData(self, *args, **kwargs):
        """
        開催情報取得

        Parameters
        ----------
        kwargs : dict
            @year
                開催年
            @month
                開催月
            @day
                カレンダーを取得する日にちの終端
        """

        # Getterクラス取得
        kwargs["getter"] = self

        # methodType指定
        kwargs["methodType"] = self.methodType.getOpenCal
        # 開催日情報をDataFrameで取得
        dfRaceIdCal = self.getRaceIdCalToDataFrame(**kwargs)

        # 取得した開催日をループして1つずつRaceIdを取り出す
        for _, item in dfRaceIdCal.iterrows():

            # open_id,racecourse_id指定
            kwargs["open_id"] = item[trn_race_id.open_id.key]
            kwargs["racecourse_id"] = item[trn_race_id.racecourse_id.key]
            if kwargs["day"] == 0:
                # methodType指定
                kwargs["methodType"] = self.methodType.getRaceId
                # レースID取得(蓄積系)
                self.getRaceIdToDataFrame(**kwargs)
            else:
                # 開催日指定
                kwargs["kaisai_date"] = (
                    str(kwargs["year"])
                    + str(kwargs["month"]).rjust(2, "0")
                    + str(kwargs["day"]).rjust(2, "0")
                )
                # methodType指定
                kwargs["methodType"] = self.methodType.getPrmRaceId
                # レースID取得(速報系)
                self.getPrmRaceIdToDataFrame(**kwargs)

    @Logger.loggerDecorator("getRaceIdDataToDataFrame")
    def getRaceIdCalToDataFrame(self, *args, **kwargs):
        """
        レースID情報取得(カレンダー取得)

        Parameters
        ----------
        kwargs : dict
            @year
                開催年度
            @month
                開催月
        """

        # 検索url(ルート)
        rootUrl: str = urlParse.urljoin(
            self.config.netkeibaUrl_race,
            self.config.netkeibaUrlSearchKeyword.race_id_cal,
        )
        # 検索url(レースID情報[カレンダー])
        raceIdCalUrl = rootUrl.format(kwargs.get("year"), kwargs.get("month"))

        # ページロード
        self.wdc.browserCtl.loadPage(raceIdCalUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)

    @Logger.loggerDecorator("getRaceIdToDataFrame")
    def getRaceIdToDataFrame(self, *args, **kwargs):
        """
        レースID情報取得(蓄積系)

        Parameters
        ----------
        kwargs : dict
            @racecourse_id
                競馬場ID
            @open_id
                開催ID
        """

        # 検索url(ルート)
        rootUrl = urlParse.urljoin(
            self.config.netkeibaUrl,
            self.config.netkeibaUrlSearchKeyword.open,
        )
        # 検索url(開催情報)
        raceIdUrl = urlParse.urljoin(
            rootUrl, kwargs.get("racecourse_id") + "/" + kwargs.get("open_id")
        )

        # ページロード
        self.wdc.browserCtl.loadPage(raceIdUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)

    @Logger.loggerDecorator("getPrmRaceIdToDataFrame")
    def getPrmRaceIdToDataFrame(self, *args, **kwargs):
        """
        レースID情報取得(速報系)

        Parameters
        ----------
        kwargs : dict
            @racecourse_id
                競馬場ID
            @open_id
                開催ID
        """

        # 検索url(ルート)
        rootUrl = urlParse.urljoin(
            self.config.netkeibaUrl_race,
            self.config.netkeibaUrlSearchKeyword.race_id_kaisai_date,
        )
        # 検索url(開催情報)
        raceIdUrl = rootUrl.format(kwargs.get("kaisai_date"))

        # ページロード
        self.wdc.browserCtl.loadPage(raceIdUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)

    def getRaceIdByLink(self, raceId_url):
        """
        リンクURLからrace_idを取得する
        
        Parameters
        ----------
        raceId_url : Any
            レースIDエレメント(a)
        """

        race_id_point = raceId_url.find("=") + 1
        return raceId_url[race_id_point : race_id_point + 12]

    class chrome(browserContainer.chrome):
        """
        ブラウザコンテナ:chrome
        """

        def __init__(
            self,
            _config: netkeiberConfig,
            _scrapingType: netkeiberConfig.settingValueStruct.ScrapingType,
        ):
            """
            コンストラクタ

            Parameters
            ----------
            _config : netkeiberConfig
                共通設定
            _scrapingType : netkeiberConfig.settingValueStruct.ScrapingType
                スクレイピングタイプ
            """

            # 基底側コンストラクタ
            super().__init__(_config, _scrapingType)

            self.cbCreateSearchResultDataFrameByWebDriver = (
                self.createSearchResultDataFrameByWebDriver
            )

        @property
        def delayWaitElement(self):
            """
            delayWaitElement
            delayWaitElementを文字列で指定する(指定したlocatorに対応するElementを指定する。未指定の場合はconfig側で指定した設定を優先する)
            """

            return "#navi_link_top"

        def getOpenCal(self, *args, **kwargs):
            """
            開催日カレンダー情報を取得する

            Parameters
            ----------
            kwargs : dict
            @day
                カレンダーを取得する日にちの終端
            """

            # getterインスタンス取得
            bc: getter_trn_race_id = kwargs.get("getter")

            # 取得対象日(下限)取得
            # (Weekly取得対応)
            targetDayFrom = int(kwargs.get("day")) - 2 if kwargs.get("day") != 0 else 0

            # bsSrc取得
            bsSrc = self.getBeautifulSoup()

            # class取得
            tables = bsSrc.find(class_="Calendar_Table").find_all(class_="RaceCellBox")

            if tables:

                # レースIDカレンダー情報model用意
                raceIdCalInfo = recset[trn_race_id](trn_race_id)

                for index in range(len(tables)):
                    try:
                        # tableTag取得
                        table_t:Tag = tables[index]

                        # 開催ID
                        open_id_a = table_t.find_all("a")
                        if open_id_a:
                            open_id_href_t:Tag = open_id_a[0]
                            open_id_href = open_id_href_t.get("href")
                            open_id = str(open_id_href).split("=")[1]
                        else:
                            continue

                        # カレンダーの日にち取得
                        targetDay = 0
                        day: str = table_t.find(class_="Day").text
                        if day.isdigit():
                            targetDay = int(day)

                        # 取得対象日(下限)と比較して対象外だった場合はスキップする
                        if targetDay < targetDayFrom:
                            continue

                        jyo_name = table_t.find_all(class_="JyoName")
                        for index in range(len(jyo_name)):
                            
                            # JyoNameエレメント取得
                            jyo_name_t:Tag = jyo_name[index]
                            
                            # 競馬場名
                            course_nm = jyo_name_t.text
                            # 競馬場ID
                            racecourse_id = bc.config.courseList[course_nm]

                            # Modelに追加
                            raceIdCalInfo.newRow()
                            raceIdCalInfo.fields(trn_race_id.race_id).value = "*"
                            raceIdCalInfo.fields(trn_race_id.racecourse_id).value = racecourse_id
                            raceIdCalInfo.fields(trn_race_id.open_id).value = open_id
                            raceIdCalInfo.fields(trn_race_id.updinfo).value = bc.getUpdInfo()

                            # コンソール出力
                            print("競馬場ID={0}".format(racecourse_id))
                            print("開催ID={0}".format(open_id))

                    except Exception as e:  # その他例外
                        Logger.logging.error(str(e))

                return raceIdCalInfo.getDataFrame()

        def getRaceId(self, *args, **kwargs):
            """
            レースID情報を取得する(蓄積系)

            Parameters
            ----------

            kwargs : dict
                @open_id
                    開催ID
                @racecourse_id
                    競馬場ID
            """

            # getterインスタンス取得
            bc: getter_trn_race_id = kwargs.get("getter")

            # 開催ID取得
            open_id: str = kwargs.get("open_id")

            # 競馬場ID取得
            racecourse_id: str = kwargs.get("racecourse_id")

            # bsSrc取得
            bsSrc = self.getBeautifulSoup()

            # class取得
            tables = bsSrc.find(class_="race_table_01").find_all("tr")

            if tables:

                # レースID情報model用意
                raceIdInfo = recset[trn_race_id](trn_race_id)

                for index in range(len(tables)):
                    if index == 0:
                        continue
                    try:
                        # tableTag取得
                        table_t:Tag = tables[index]
                        
                        # tdTag取得
                        row = table_t.find_all("td")

                        # レースID
                        row_t:Tag = row[1]
                        race_id = str(row_t.find_all("a")[0].get("href")).split("/")[2]

                        # レースIDが数値で構成されていなければ例外を発生させる
                        if not race_id.isdigit():
                            raise racdIdCheckError

                        # Modelに追加
                        raceIdInfo.newRow()
                        raceIdInfo.fields(trn_race_id.race_id).value = race_id
                        raceIdInfo.fields(trn_race_id.racecourse_id).value = racecourse_id
                        raceIdInfo.fields(trn_race_id.open_id).value = open_id
                        raceIdInfo.fields(trn_race_id.scraping_count).value = 0
                        raceIdInfo.fields(trn_race_id.get_time).value = 0
                        raceIdInfo.fields(trn_race_id.get_status).value = nd.getStatus.unacquired.value
                        raceIdInfo.fields(trn_race_id.updinfo).value = bc.getUpdInfo()

                        # コンソール出力
                        print("レースID={0}".format(race_id))
                        print("競馬場ID={0}".format(racecourse_id))
                        print("開催日={0}".format(open_id))

                    except racdIdCheckError as e:
                        Logger.logging.error(str(e))
                        Logger.logging.error("race_id Value={0}".format(race_id))
                        Logger.logging.error("open_id Value={0}".format(open_id))

                    except Exception as e:  # その他例外
                        Logger.logging.error(str(e))

                # レコードセットマージ
                bc.rsRaceId.merge(raceIdInfo, False)

                # 戻り値をDataFrameで返却
                return raceIdInfo.getDataFrame()

        def getPrmRaceId(self, *args, **kwargs):
            """
            レースID情報を取得する(速報系)

            Parameters
            ----------
            kwargs : dict
                @open_id
                    開催ID
                @racecourse_id
                    競馬場ID
            """

            # getterインスタンス取得
            bc: getter_trn_race_id = kwargs.get("getter")

            # 開催ID取得
            open_id: str = kwargs.get("open_id")

            # 競馬場ID取得
            racecourse_id: str = kwargs.get("racecourse_id")

            # レースID情報model用意
            raceIdInfo = recset[trn_race_id](trn_race_id)

            # element取得
            elements = self.wDriver.find_elements_by_class_name("RaceList_DataItem")
            for elm in elements:

                try:

                    # aタグ取得
                    elm_we: WebElement = elm
                    elems_a: WebElement = elm_we.find_element_by_tag_name("a")

                    # race_id取得
                    race_id:str = bc.getRaceIdByLink(elems_a.get_attribute("href"))

                    # レースIDが数値で構成されていなければ例外を発生させる
                    if not race_id.isdigit():
                        raise racdIdCheckError

                    # Modelに追加
                    raceIdInfo.newRow()
                    raceIdInfo.fields(trn_race_id.race_id).value = race_id
                    raceIdInfo.fields(trn_race_id.racecourse_id).value = racecourse_id
                    raceIdInfo.fields(trn_race_id.open_id).value = open_id
                    raceIdInfo.fields(trn_race_id.scraping_count).value = 0
                    raceIdInfo.fields(trn_race_id.get_time).value = 0
                    raceIdInfo.fields(trn_race_id.get_status).value = nd.getStatus.unacquired.value
                    raceIdInfo.fields(trn_race_id.updinfo).value = bc.getUpdInfo()

                    # コンソール出力
                    print("レースID={0}".format(race_id))
                    print("競馬場ID={0}".format(racecourse_id))
                    print("開催日={0}".format(open_id))

                except racdIdCheckError as e:
                    Logger.logging.error(str(e))
                    Logger.logging.error("race_id Value={0}".format(race_id))
                    Logger.logging.error("open_id Value={0}".format(open_id))

                except Exception as e:  # その他例外
                    Logger.logging.error(str(e))

            # レコードセットマージ
            bc.rsRaceId.merge(raceIdInfo, False)

            # 戻り値をDataFrameで返却
            return raceIdInfo.getDataFrame()

        def createSearchResultDataFrameByWebDriver(
            self, _, *args, **kwargs
        ) -> DataFrame:
            """
            レースID情報をDataFrameで返す(By Selenium)
            """

            bc: getter_trn_race_id = kwargs.get("getter")
            if kwargs["methodType"] == bc.methodType.getOpenCal:
                return self.getOpenCal(*args, **kwargs)
            elif kwargs["methodType"] == bc.methodType.getRaceId:
                return self.getRaceId(*args, **kwargs)
            elif kwargs["methodType"] == bc.methodType.getPrmRaceId:
                return self.getPrmRaceId(*args, **kwargs)



from LibHanger.Models.recset import recset
from netkeiber.Library.netkeiberController import netkeibaBrowserController
from netkeiber.Library.netkeiberGlobals import *

class baseGetter(netkeibaBrowserController):
    """
    Getter基底
    """

    __dictResult = {}
    """ 処理結果 """

    def __init__(self) -> None:
        """
        コンストラクタ
        """

        # 基底コンストラクタ
        super().__init__(gv.netkeiberConfig)

        # 処理結果Dict
        self.__dictResult = {}

    @property
    def scrapingCount(self):
        """
        スクレイピング回数
        """

        return (
            self.wdc.browserCtl.loadPageCount if not self.wdc.browserCtl is None else 0
        )

    class getterResult:
        """
        Getter処理結果
        """

        def __init__(self, __recordCount: int, __recSet: recset):
            """
            コンストラクタ

            Parameters
            ----------
            __recordCount : int
                レコード数
            __recSet : recset
                レコードセット
            """

            self.recordCount: int = __recordCount
            """ 取得件数 """

            self.recSet: recset = __recSet
            """ 取得したレコードセット """

    def addResult(self, __key, __recordCount: int, __recSet: recset):
        """
        処理結果追加

        Parameters
        ----------
        __key : Any
            処理結果Key(テーブル名)
        __recordCount : int
            レコード数
        __recSet : recset
            レコードセット
        """

        # 処理結果をdictにセット
        self.__dictResult[__key] = self.getterResult(__recordCount, __recSet)

        # 戻り値を返す
        return self.__dictResult



import urllib.parse as urlParse
import LibHanger.Library.uwLogger as Logger
from pandas.core.frame import DataFrame
from bs4 import Tag
from LibHanger.Models.recset import recset
from Scrapinger.Library.browserContainer import browserContainer
from netkeiber.Library.netkeiberConfig import netkeiberConfig
from netkeiber.Library.netkeiberDeclare import netkeiberDeclare as nd
from netkeiber.Library.netkeiberException import getterError
from netkeiber.Library.netkeiberException import gettingValueError
from netkeiber.Models.trn_refund_info import trn_refund_info
from netkeiber.Getter.Base.baseGetter import baseGetter


class getter_trn_refund_info(baseGetter):
    """
    払戻情報取得クラス
    (trn_refund_info)
    """

    class payInfo:
        """
        払戻情報
        """

        seq: int = 0
        """ 連番 """

        horse_no: str = ""
        """ 馬番 """

        pay: int = 0
        """ 払戻金 """

        pop: int = 0
        """ 人気 """

    def __init__(self) -> None:
        """
        コンストラクタ
        """

        super().__init__()

        # レコードセット初期化
        self.init_recset()

    def init_recset(self):
        """
        レコードセット初期化
        """

        # レコードセット初期化
        self.rsRefundInfo = recset[trn_refund_info](trn_refund_info)

    @property
    def scrapingType(self):
        """
        ScrapingType
        スクレイピング方法を指定する(unknownの場合はconfig側で指定した設定を優先する)
        """

        return netkeiberConfig.settingValueStruct.ScrapingType.selenium

    @Logger.loggerDecorator("getData", ["refund_id", "racecourse_id"])
    def getData(self, *args, **kwargs):
        """
        払戻情報取得

        Parameters
        ----------
        kwargs : dict
            @refund_id
                払戻ID
        """

        # 払戻情報をDataFrameで取得
        try:
            kwargs["getter"] = self
            result = self.getRefundInfoDataToDataFrame(**kwargs)
        except:
            raise getterError
        return result

    @Logger.loggerDecorator("getRefundInfoDataToDataFrame")
    def getRefundInfoDataToDataFrame(self, *args, **kwargs):
        """
        払戻情報取得

        Parameters
        ----------
        kwargs : dict
            @refund_id
                払戻ID
        """

        # 検索url(ルート)
        rootUrl = urlParse.urljoin(
            self.config.netkeibaUrl,
            self.config.netkeibaUrlSearchKeyword.refund,
        )
        # 検索url(払戻情報)
        refundInfoUrl = urlParse.urljoin(
            rootUrl, kwargs.get("racecourse_id") + "/" + kwargs.get("refund_id")
        )

        # ページロード
        self.wdc.browserCtl.loadPage(refundInfoUrl)

        # pandasデータを返却する
        return self.wdc.browserCtl.createSearchResultDataFrame(**kwargs)

    def getHorseNo(self, horseNoString: str) -> str:
        """
        馬番取得

        Parameters
        ----------
        horseNoString : str
            馬番(文字列)
        """

        return horseNoString.replace(" ", "")

    def getPay(self, payString: str) -> int:
        """
        払戻金取得

        Parameters
        ----------
        payString : str
            払戻金(文字列)
        """

        pay = 0

        try:
            pay = int(payString.replace(",", "")) if payString != "" else 0
        except:
            raise gettingValueError

        return pay

    def getPop(self, popString: str) -> int:
        """
        人気取得

        Parameters
        ----------
        popString : str
            人気(文字列)
        """

        pop = 0

        try:
            pop = int(popString.replace(" ", "")) if popString != "" else 0
        except:
            raise gettingValueError

        return pop

    def addRecsetPayInfo(
        self,
        refund_id,
        racecourse_id,
        race_no,
        refund_kbn,
        piList: list[payInfo],
        refundInfo: recset,
    ):
        """
        払戻情報をレコードセットに追加

        Parameters
        ----------
        refund_id : str
            払戻ID
        racecourse_id : str
            競馬場ID
        race_no : int
            レース番号
        refund_kbn : str
            払戻区分
        piList : list[payInfo]
            払戻情報リスト
        refundInfo : recset
            払戻情報レコードセット

        """

        for pi in piList:
            refundInfo.newRow()
            refundInfo.fields(trn_refund_info.refund_id).value = refund_id
            refundInfo.fields(trn_refund_info.racecourse_id).value = racecourse_id
            refundInfo.fields(trn_refund_info.race_no).value = race_no
            refundInfo.fields(trn_refund_info.refund_kbn).value = refund_kbn
            refundInfo.fields(trn_refund_info.refund_seq).value = pi.seq
            refundInfo.fields(trn_refund_info.horse_no).value = pi.horse_no
            refundInfo.fields(trn_refund_info.pay).value = pi.pay
            refundInfo.fields(trn_refund_info.pop).value = pi.pop
            refundInfo.fields(trn_refund_info.updinfo).value = self.getUpdInfo()

    def getPayInfoSingle(self, payInfoTable):
        """
        払戻情報取得(単)
        
        Parameters
        ----------
        payInfoTable : Any
            払戻情報テーブル
        """

        # 払戻情報初期化
        payInfoSingle = [self.payInfo]
        payInfoSingle.clear()
        
        # 馬番、払戻金額、人気Tagを取得
        horseNo_t:Tag = payInfoTable[0]
        pay_t:Tag = payInfoTable[1]
        pop_t:Tag = payInfoTable[2]
        
        # 払戻テーブルのループ回数取得
        loopCount = len(horseNo_t.contents)

        # 連番初期化
        seq = 0
        for contents_idx in range(0, loopCount, 1):
            
            if horseNo_t.contents[contents_idx].name == "br":
                continue

            pi = self.payInfo()

            # 連番
            seq += 1
            pi.seq = seq
            # 馬番
            pi.horse_no = self.getHorseNo(str(horseNo_t.contents[contents_idx]))
            # 払戻
            pi.pay = self.getPay(str(pay_t.contents[contents_idx]))
            # 人気
            pi.pop = self.getPop(str(pop_t.contents[contents_idx]))

            # 払戻情報をリストに追加
            payInfoSingle.append(pi)

        # 戻り値を返す
        return payInfoSingle

    def getPayInfoDouble(self, payInfoTable):
        """
        払戻情報取得(複)

        Parameters
        ----------
        payInfoTable : Any
            払戻情報テーブル
        """

        # 馬番、払戻金額、人気Tagを取得
        horseNo_t:Tag = payInfoTable[0]
        pay_t:Tag = payInfoTable[1]
        pop_t:Tag = payInfoTable[2]

        # 1組目
        horse_no1: str = self.getHorseNo(str(horseNo_t.contents[0]))
        pay1: int = self.getPay(str(pay_t.contents[0]))
        pop1: int = self.getPop(str(pop_t.contents[0]))

        # 2組目
        horse_no2: str = self.getHorseNo(str(horseNo_t.contents[2]))
        pay2: int = self.getPay(str(pay_t.contents[2]))
        pop2: int = self.getPop(str(pop_t.contents[2]))

        # 3組目
        horse_no3: str = self.getHorseNo(
            str(horseNo_t.contents[4])
            if len(horseNo_t.contents) > 4
            else ""
        )
        pay3: int = self.getPay(
            str(pay_t.contents[4])
            if len(pay_t.contents) > 4
            else ""
        )
        pop3: int = self.getPop(
            str(pop_t.contents[4])
            if len(pop_t.contents) > 4
            else ""
        )

        # 戻り値を返す
        return horse_no1, pay1, pop1, horse_no2, pay2, pop2, horse_no3, pay3, pop3

    class chrome(browserContainer.chrome):
        """
        ブラウザコンテナ:chrome
        """

        def __init__(
            self,
            _config: netkeiberConfig,
            _scrapingType: netkeiberConfig.settingValueStruct.ScrapingType,
        ):
            """
            コンストラクタ

            Parameters
            ----------
            _config : netkeiberConfig
                共通設定
            _scrapingType : netkeiberConfig.settingValueStruct.ScrapingType
                スクレイピングタイプ
            """

            # 基底側コンストラクタ
            super().__init__(_config, _scrapingType)

            self.cbCreateSearchResultDataFrameByWebDriver = (
                self.createSearchResultDataFrameByWebDriver
            )

        @property
        def delayWaitElement(self):
            """
            delayWaitElement
            delayWaitElementを文字列で指定する(指定したlocatorに対応するElementを指定する。未指定の場合はconfig側で指定した設定を優先する)
            """

            return "#contents"

        def createSearchResultDataFrameByWebDriver(
            self, _, *args, **kwargs
        ) -> DataFrame:
            """
            払戻情報をDataFrameで返す(By Selenium)
            """

            return self.getRefundInfo(*args, **kwargs)

        def getRefundInfo(self, *args, **kwargs):
            """
            払戻情報を取得する

            Parameters
            ----------
            kwargs : dict
                @refund_id
                    取得対象払戻ID
                @racecourse_id
                    競馬場ID
            """
            
            # 払戻ID取得
            refund_id: str = kwargs.get("refund_id")
            # 競馬場ID取得
            racecourse_id: str = kwargs.get("racecourse_id")

            # getterインスタンス取得
            bc: getter_trn_refund_info = kwargs.get("getter")

            # 払戻テーブル項目名
            pil = bc.config.settingValueStruct.PayInfoClass

            # bsSrc取得
            bsSrc = self.getBeautifulSoup()

            # class取得
            tables = bsSrc.find_all(class_="race_payback_info")

            if tables:

                # 払戻情報model用意
                refundInfo = recset[trn_refund_info](trn_refund_info)

                for tables_idx in range(len(tables)):

                    try:

                        # 払戻情報テーブルクラス取得
                        refundInfoTbl: Tag = tables[tables_idx]

                        # dtタグ検索
                        dt = refundInfoTbl.find("dt")
                        # レース番号取得
                        race_no = 0
                        if len(dt.contents) > 0:
                            race_no = int(dt.contents[0].text.replace("R", ""))
                        else:
                            raise gettingValueError

                        # 払戻情報テーブルクラス取得
                        payInfo = refundInfoTbl.find_all(class_="pay_table_01")

                        # 払戻情報テーブルには[pay_table_01]クラスが2つ存在する(1つの場合もある)ので
                        # ループしてそれぞれの払戻種類を取得する
                        for payInfo_idx in range(len(payInfo)):

                            # trタグ取得
                            payInfoTbl: Tag = payInfo[payInfo_idx]
                            payInfoTables = payInfoTbl.find_all("tr")
                            for tableIndex in range(len(payInfoTables)):

                                # 払戻情報テーブル取得
                                payInfoRow: Tag = payInfoTables[tableIndex]
                                payInfoTable = payInfoRow.find_all("td")

                                # 単勝
                                tan = payInfoRow.find(class_=pil.win)
                                if tan:

                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(
                                        refund_id,
                                        racecourse_id,
                                        race_no,
                                        nd.refundKbn.win.value,
                                        piList,
                                        refundInfo,
                                    )

                                # 複勝
                                fuku = payInfoRow.find(class_=pil.dwin)
                                if fuku:

                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(
                                        refund_id,
                                        racecourse_id,
                                        race_no,
                                        nd.refundKbn.dwin.value,
                                        piList,
                                        refundInfo,
                                    )
                                # 枠連
                                waku = payInfoRow.find(class_=pil.fcwin)
                                if waku:

                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(
                                        refund_id,
                                        racecourse_id,
                                        race_no,
                                        nd.refundKbn.fcwin.value,
                                        piList,
                                        refundInfo,
                                    )

                                # 馬連
                                uren = payInfoRow.find(class_=pil.hcwin)
                                if uren:

                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(
                                        refund_id,
                                        racecourse_id,
                                        race_no,
                                        nd.refundKbn.hcwin.value,
                                        piList,
                                        refundInfo,
                                    )

                                # ワイド
                                wide = payInfoRow.find(class_=pil.wdwin)
                                if wide:

                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(
                                        refund_id,
                                        racecourse_id,
                                        race_no,
                                        nd.refundKbn.wdwin.value,
                                        piList,
                                        refundInfo,
                                    )

                                # 馬単
                                utan = payInfoRow.find(class_=pil.hswin)
                                if utan:

                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(
                                        refund_id,
                                        racecourse_id,
                                        race_no,
                                        nd.refundKbn.hswin.value,
                                        piList,
                                        refundInfo,
                                    )

                                # 三連複
                                sanfuku = payInfoRow.find(class_=pil.tdwin)
                                if sanfuku:

                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(
                                        refund_id,
                                        racecourse_id,
                                        race_no,
                                        nd.refundKbn.tdwin.value,
                                        piList,
                                        refundInfo,
                                    )

                                # 三連単
                                santan = payInfoRow.find(class_=pil.tswin)
                                if santan:

                                    # 馬番,払戻,人気
                                    piList = bc.getPayInfoSingle(payInfoTable)

                                    # レコードセットに追加
                                    bc.addRecsetPayInfo(
                                        refund_id,
                                        racecourse_id,
                                        race_no,
                                        nd.refundKbn.tswin.value,
                                        piList,
                                        refundInfo,
                                    )

                    except gettingValueError as gvException:  # 値例外
                        Logger.logging.error(str(gvException))
                        raise
                    except Exception as e:  # その他例外
                        Logger.logging.error(str(e))
                        raise

            # コンソール出力
            print("払戻ID={0}".format(refund_id))

            # レコードセットマージ
            bc.rsRefundInfo.merge(refundInfo)

            # 戻り値を返す
            return refundInfo.getDataFrame()
        